Package: parsetools
Type: Package
Title: Parse Tools
Version: 0.1.3
Authors@R: c(
    person("Andrew", "Redd", , "andrew.redd@hsc.utah.edu", c("aut", "cre")),
    person("R Consortium", role=c("cph", "fnd"))
    )
Maintainer: Andrew Redd <andrew.redd@hsc.utah.edu>
License: GPL-2
Language: en-US
Imports: methods, utils
Suggests: covr, knitr, rmarkdown, testthat
Description: Tools and utilities for dealing with parse data. 
    Parse data represents the parse tree as data with location and type
    information.  This package provides functions for navigating the  
    parse tree as a data frame.
Collate: 'internal.R' 'accessors.R' 'checks.R' 'children.R'
        'comments.R' 'errors.R' 'family.R' 'find-utils.R' 'firstborn.R'
        'get_parse_data.R' 'grouping.R' 'iff_blocks.R' 'parent.R'
        'pd_assign.R' 'pd_call.R' 'pd_make_is_in.R' 'pd_classes.R'
        'pd_comments.R' 'pd_function.R' 'pd_identify.R' 'pd_if.R'
        'reconstitute.R' 'root.R' 'siblings.R' 'tags.R'
        'testing_blocks.R' 'zzz.R'
Encoding: UTF-8
RoxygenNote: 7.0.2
VignetteBuilder: knitr
KeepSource: yes
URL: https://github.com/RDocTaskForce/parsetools
BugReports: https://github.com/RDocTaskForce/parsetools/issues
NeedsCompilation: no
Packaged: 2020-04-07 22:45:53 UTC; u0092104
Author: Andrew Redd [aut, cre],
  R Consortium [cph, fnd]
Repository: CRAN
Date/Publication: 2020-04-08 16:30:02 UTC
