# Generated by roxygen2: do not edit by hand

S3method("-","parse-data")
S3method("[","parse-data")
S3method(as.data.frame,parseData)
S3method(get_parse_data,"function")
S3method(get_parse_data,default)
S3method(get_parse_data,srcfile)
S3method(get_parse_data,srcref)
S3method(pd_identify,"NULL")
S3method(pd_identify,default)
S3method(pd_identify,srcref)
S3method(sort,"parse-data")
S3method(strip_doc_comment_leads,character)
S3method(strip_doc_comment_leads,data.frame)
S3method(subset,"parse-data")
export(extract_test_block)
export(extract_test_blocks)
export(get_parse_data)
export(pd_get_comment_tag_content)
export(pd_identify)
export(strip_doc_comment_leads)
export(valid_parse_data)
exportPattern("^pd_.*")
