% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tags.R
\name{pd_has_tag}
\alias{pd_has_tag}
\title{Check if there is a documentation \code{@} tag.}
\usage{
pd_has_tag(id, pd, tag, ...)
}
\arguments{
\item{id}{id of the expression of interest}

\item{pd}{The \code{\link{parse-data}} information}

\item{tag}{tag(s) to test for}

\item{...}{options passed on}
}
\description{
Check if a node of \code{parse-data} identified by \code{id}
is both a comment and contains a documentation tag identified by
the \code{@} symbol.
}
